@@ -3,14 +3,13 @@ from argparse import ArgumentParser
 from copy import copy
 from functools import partial
 from queue import PriorityQueue
-from typing import List, Tuple, Optional, Dict
+from typing import List, Tuple, Optional, Dict, Any
 
 import numpy as np
 from numpy.random import RandomState
 
 from rlai.actions import Action
 from rlai.agents import Agent
-from rlai.agents.mdp import MdpAgent
 from rlai.environments import Environment
 from rlai.meta import rl_text
 from rlai.planning.environment_models import StochasticEnvironmentModel
@@ -29,12 +28,13 @@ class MdpEnvironment(Environment, ABC):
 
     def reset_for_new_run(
             self,
-            agent: MdpAgent
-    ) -> State:
+            agent: Any
+    ) -> MdpState:
         """
         Reset the the environment to a random nonterminal state, if any are specified, or to None.
 
         :param agent: Agent used to generate on-the-fly state identifiers.
+        :return: Initial state.
         """
 
         super().reset_for_new_run(agent)
@@ -195,13 +195,13 @@ class ModelBasedMdpEnvironment(MdpEnvironment, ABC):
         ])
 
         # sample next state and reward
-        next_state, next_reward = sample_list_item(
+        self.state, next_reward = sample_list_item(
             x=s_prime_rewards,
             probs=probs,
             random_state=self.random_state
         )
 
-        return next_state, next_reward
+        return self.state, next_reward
 
     def __init__(
             self,
@@ -381,7 +381,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
 
         parsed_args, unparsed_args = parse_arguments(cls, args)
 
-        planning_environment = PrioritizedSweepingMdpPlanningEnvironment(
+        planning_environment = cls(
             name=f"prioritized planning (theta={parsed_args.priority_theta})",
             random_state=random_state,
             model=StochasticEnvironmentModel(),
@@ -395,7 +395,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
             state: MdpState,
             t: int,
             a: Action,
-            agent: MdpAgent
+            agent: Any
     ) -> Tuple[Tuple[MdpState, Action, MdpState], Reward]:
         """
         Advance a planning state based on priorities.
@@ -413,13 +413,13 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
         if planning_state is None:
             planning_state = state
             planning_a = a
-            next_state = copy(state)
-            next_state.terminal = True
+            self.state = copy(state)
+            self.state.terminal = True
             r = 0.0
         else:
 
             # sample next state and reward from model
-            next_state, r = self.model.sample_next_state_and_reward(planning_state, planning_a, self.random_state)
+            self.state, r = self.model.sample_next_state_and_reward(planning_state, planning_a, self.random_state)
 
             # add predecessors into priority queue
             for pred_state, pred_action, r in self.get_predecessor_state_action_rewards(planning_state):
@@ -427,7 +427,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
                 priority = -abs(target_value - self.q_S_A[pred_state][pred_action].get_value())
                 self.add_state_action_priority(pred_state, pred_action, priority)
 
-        return (planning_state, planning_a, next_state), Reward(None, r)
+        return (planning_state, planning_a, self.state), Reward(None, r)
 
     def get_predecessor_state_action_rewards(
             self,
@@ -534,16 +534,16 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :return: State dictionary.
         """
 
-        state_dict = dict(self.__dict__)
+        state = dict(self.__dict__)
 
         # the priority queue cannot be pickled. blank it out.
-        state_dict['state_action_priority'] = None
+        state['state_action_priority'] = None
 
-        return state_dict
+        return state
 
     def __setstate__(
             self,
-            state
+            state: Dict
     ):
         """
         Set the state dictionary.
@@ -598,8 +598,8 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
 
         parsed_args, unparsed_args = parse_arguments(cls, args)
 
-        planning_environment = TrajectorySamplingMdpPlanningEnvironment(
-            name=f"trajectory planning",
+        planning_environment = cls(
+            name='trajectory planning',
             random_state=random_state,
             model=StochasticEnvironmentModel(),
             **vars(parsed_args)
@@ -612,7 +612,7 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
             state: MdpState,
             t: int,
             a: Action,
-            agent: MdpAgent
+            agent: Any
     ) -> Tuple[Tuple[MdpState, Action, MdpState], Reward]:
         """
         Advance a planning state.
@@ -629,9 +629,9 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
             a = self.model.sample_action(state, self.random_state)
 
         # sample next state and reward from model
-        next_state, r = self.model.sample_next_state_and_reward(state, a, self.random_state)
+        self.state, r = self.model.sample_next_state_and_reward(state, a, self.random_state)
 
-        return (state, a, next_state), Reward(None, r)
+        return (state, a, self.state), Reward(None, r)
 
     def __init__(
             self,
@@ -659,3 +659,50 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
             model=model,
             num_planning_improvements_per_direct_improvement=num_planning_improvements_per_direct_improvement
         )
+
+
+@rl_text(chapter='Environments', page=1)
+class ContinuousMdpEnvironment(MdpEnvironment, ABC):
+    """
+    MDP environment in which states and actions are continuous and multidimensional.
+    """
+
+    @abstractmethod
+    def get_state_space_dimensionality(
+            self
+    ) -> int:
+        """
+        Get the dimensionality of the state space.
+
+        :return: Number of dimensions.
+        """
+
+    @abstractmethod
+    def get_state_dimension_names(
+            self
+    ) -> List[str]:
+        """
+        Get names of state dimensions.
+
+        :return: List of names.
+        """
+
+    @abstractmethod
+    def get_action_space_dimensionality(
+            self
+    ) -> int:
+        """
+        Get the dimensionality of the action space.
+
+        :return: Number of dimensions.
+        """
+
+    @abstractmethod
+    def get_action_dimension_names(
+            self
+    ) -> List[str]:
+        """
+        Get action names.
+
+        :return: List of names.
+        """
