@@ -8,6 +8,7 @@ from typing import List, Tuple, Optional, Union, Dict
 
 import gym
 import numpy as np
+from PyQt5.QtWidgets import QApplication
 from gym.envs.registration import EnvSpec
 from gym.spaces import Discrete, Box
 from gym.wrappers import TimeLimit
@@ -79,6 +80,8 @@ class Gym(ContinuousMdpEnvironment):
     MCC_V0_GOAL_X_POS = 0.45
     MCC_V0_FUEL_CONSUMPTION_FULL_THROTTLE = 1.0 / 300.0
 
+    SWIMMER_V2 = 'Swimmer-v2'
+
     @classmethod
     def get_argument_parser(
             cls,
@@ -283,6 +286,10 @@ class Gym(ContinuousMdpEnvironment):
             if self.plot_environment:
                 self.state_reward_scatter_plot.update(np.append(observation, reward))
 
+            # swimmer is a non-qt environment, so we need to process qt events manually.
+            if self.gym_id == Gym.SWIMMER_V2:
+                QApplication.processEvents()
+
         self.state = GymState(
             environment=self,
             observation=observation,
@@ -336,9 +343,9 @@ class Gym(ContinuousMdpEnvironment):
         """
         Check whether the current episode is to be rendered.
 
-        :param render_manually: Whether the rendering will be done manually with calls to the render function or automatically
-        as a result of saving videos via the monitor. Pass None to check whether the episode should be rendered,
-        regardless of how the rendering will be done.
+        :param render_manually: Whether the rendering will be done manually with calls to the render function or
+        automatically as a result of saving videos via the monitor. Pass None to check whether the episode should be
+        rendered, regardless of how the rendering will be done.
         :return: True if rendered and False otherwise.
         """
 
@@ -459,7 +466,7 @@ class Gym(ContinuousMdpEnvironment):
             self
     ) -> List[str]:
         """
-        Get action names.
+        Get names of action dimensions.
 
         :return: List of names.
         """
@@ -474,7 +481,7 @@ class Gym(ContinuousMdpEnvironment):
                 'throttle'
             ]
         else:  # pragma no cover
-            warnings.warn(f'The action names for {self.gym_id} are unknown. Defaulting to numbers.')
+            warnings.warn(f'The action dimension names for {self.gym_id} are unknown. Defaulting to numbers.')
             names = [str(x) for x in range(0, self.get_action_space_dimensionality())]
 
         return names
@@ -847,6 +854,59 @@ class ContinuousFeatureExtractor(StateFeatureExtractor):
             )
 
 
+@rl_text(chapter='Feature Extractors', page=1)
+class SignedCodingFeatureExtractor(ContinuousFeatureExtractor):
+    """
+    Signed-coding feature extractor. Forms a category from the conjunction of all state-feature signs and then places
+    the continuous feature vector into its associated category.
+    """
+
+    def extract(
+            self,
+            state: GymState,
+            refit_scaler: bool
+    ) -> np.ndarray:
+        """
+        Extract state features.
+
+        :param state: State.
+        :param refit_scaler: Whether to refit the feature scaler before scaling the extracted features.
+        :return: State-feature vector.
+        """
+
+        if self.state_category_interacter is None:
+            self.state_category_interacter = OneHotCategoricalFeatureInteracter([
+                OneHotCategory(*category_args)
+                for category_args in product(*([[True, False]] * state.observation.shape[0]))
+            ])
+
+        # form the one-hot state category
+        state_category = OneHotCategory(*[
+            value < 0.0
+            for value in state.observation
+        ])
+
+        # extract and encode feature values
+        raw_feature_values = super().extract(state, refit_scaler)
+        encoded_feature_values = self.state_category_interacter.interact(
+            np.array([raw_feature_values]),
+            [state_category]
+        )[0]
+
+        return encoded_feature_values
+
+    def __init__(
+            self
+    ):
+        """
+        Initialize the feature extractor.
+        """
+
+        super().__init__()
+
+        self.state_category_interacter = None
+
+
 @rl_text(chapter='Feature Extractors', page=1)
 class ContinuousLunarLanderFeatureExtractor(ContinuousFeatureExtractor):
     """
@@ -920,71 +980,6 @@ class ContinuousLunarLanderFeatureExtractor(ContinuousFeatureExtractor):
         ])
 
 
-@rl_text(chapter='Feature Extractors', page=1)
-class WalkerFeatureExtractor(ContinuousFeatureExtractor):
-    """
-    Feature extractor for the walker.
-    """
-
-    def extract(
-            self,
-            state: GymState,
-            refit_scaler: bool
-    ) -> np.ndarray:
-        """
-        Extract state features.
-
-        :param state: State.
-        :param refit_scaler: Whether or not to refit the feature scaler before scaling the extracted features.
-        :return: State-feature vector.
-        """
-
-        # extract raw feature values
-        raw_feature_values = super().extract(state, refit_scaler)
-
-        # features:
-        #
-
-        # # form the one-hot state category. start by thresholding some feature values.
-        # state_category_feature_idxs = list(range(raw_feature_values.shape[0]))
-        # state_category = OneHotCategory(*[
-        #     value < 0.0
-        #     for idx, value in zip(state_category_feature_idxs, state.observation[state_category_feature_idxs])
-        # ])
-        #
-        # # encode feature values
-        # encoded_feature_idxs = state_category_feature_idxs
-        # feature_values_to_encode = raw_feature_values[encoded_feature_idxs]
-        # encoded_feature_values = self.state_category_interacter.interact(
-        #     np.array([feature_values_to_encode]),
-        #     [state_category]
-        # )[0]
-
-        # # get unencoded feature values
-        # both_legs_in_contact = 1.0 if all(raw_feature_values[6:8] == 1.0) else 0.0
-        # unencoded_feature_values = np.append(raw_feature_values[[1, 6, 7, 8]], [both_legs_in_contact])
-        #
-        # # combine encoded and unencoded feature values
-        # final_feature_values = np.append(encoded_feature_values, unencoded_feature_values)
-
-        return raw_feature_values
-
-    def __init__(
-            self
-    ):
-        """
-        Initialize the feature extractor.
-        """
-
-        super().__init__()
-
-        # interact features with relevant state categories
-        self.state_category_interacter = OneHotCategoricalFeatureInteracter([
-            OneHotCategory(*category_args)
-            for category_args in product(*([[True, False]] * 5))
-        ])
-
-
 @rl_text(chapter='Feature Extractors', page=1)
 class ContinuousMountainCarFeatureExtractor(ContinuousFeatureExtractor):
     """
