@@ -8,17 +8,17 @@ from PyQt5.QtWidgets import QApplication
 from numpy.random import RandomState
 
 from rlai.actions import Action, ContinuousMultiDimensionalAction
-from rlai.agents import Agent
-from rlai.agents.mdp import MdpAgent, StochasticMdpAgent
+from rlai.agents.mdp import MdpAgent, ParameterizedMdpAgent
 from rlai.environments import Environment
 from rlai.environments.mdp import ContinuousMdpEnvironment
 from rlai.environments.network import TcpMdpEnvironment
 from rlai.meta import rl_text
 from rlai.models.feature_extraction import FeatureExtractor, NonstationaryFeatureScaler
-from rlai.policies import Policy
+from rlai.policies.parameterized import ParameterizedPolicy
 from rlai.rewards import Reward
 from rlai.states.mdp import MdpState
 from rlai.utils import parse_arguments
+from rlai.v_S import StateValueEstimator
 from rlai.v_S.function_approximation.models.feature_extraction import StateFeatureExtractor
 
 
@@ -63,7 +63,7 @@ class RobocodeReward(Reward):
 
 
 @rl_text(chapter='Agents', page=1)
-class RobocodeAgent(StochasticMdpAgent):
+class RobocodeAgent(ParameterizedMdpAgent):
     """
     Robocode agent.
     """
@@ -87,33 +87,6 @@ class RobocodeAgent(StochasticMdpAgent):
 
         return parser
 
-    @classmethod
-    def init_from_arguments(
-            cls,
-            args: List[str],
-            random_state: RandomState,
-            pi: Optional[Policy]
-    ) -> Tuple[List[Agent], List[str]]:
-        """
-        Initialize a Robocode agent from arguments.
-
-        :param args: Arguments.
-        :param random_state: Random state.
-        :param pi: Policy.
-        :return: 2-tuple of a list of agents and a list of unparsed arguments.
-        """
-
-        parsed_args, unparsed_args = parse_arguments(cls, args)
-
-        agent = cls(
-            name=f'Robocode (gamma={parsed_args.gamma})',
-            random_state=random_state,
-            pi=pi,
-            **vars(parsed_args)
-        )
-
-        return [agent], unparsed_args
-
     def shape_reward(
             self,
             reward: Reward,
@@ -188,8 +161,9 @@ class RobocodeAgent(StochasticMdpAgent):
             self,
             name: str,
             random_state: RandomState,
-            pi: Policy,
-            gamma: float
+            pi: ParameterizedPolicy,
+            gamma: float,
+            v_S: StateValueEstimator
     ):
         """
         Initialize the agent.
@@ -198,13 +172,15 @@ class RobocodeAgent(StochasticMdpAgent):
         :param random_state: Random state.
         :param pi: Policy.
         :param gamma: Discount.
+        :param v_S: Baseline state-value estimator.
         """
 
         super().__init__(
             name=name,
             random_state=random_state,
             pi=pi,
-            gamma=gamma
+            gamma=gamma,
+            v_S=v_S
         )
 
 
