@@ -27,7 +27,7 @@ def evaluate_v_pi(
     :param agent: MDP agent. Must contain a policy `pi` that has been fully initialized with instances of
     `rlai.states.mdp.ModelBasedMdpState`.
     :param environment: Model-based MDP environment to evaluate.
-    :param theta: Minimum tolerated change in state value estimates, below which evaluation terminates. Either `theta`
+    :param theta: Minimum tolerated change in state-value estimates, below which evaluation terminates. Either `theta`
     or `num_iterations` (or both) can be specified, but passing neither will raise an exception.
     :param num_iterations: Number of evaluation iterations to execute.  Either `theta` or `num_iterations` (or both)
     can be specified, but passing neither will raise an exception.
@@ -113,7 +113,7 @@ def evaluate_q_pi(
 
     :param agent: MDP agent.
     :param environment: Model-based MDP environment to evaluate.
-    :param theta: Minimum tolerated change in state value estimates, below which evaluation terminates. Either `theta`
+    :param theta: Minimum tolerated change in state-value estimates, below which evaluation terminates. Either `theta`
     or `num_iterations` (or both) can be specified, but passing neither will raise an exception.
     :param num_iterations: Number of evaluation iterations to execute.  Either `theta` or `num_iterations` (or both)
     can be specified, but passing neither will raise an exception.
@@ -151,7 +151,7 @@ def evaluate_q_pi(
             q_S_A_to_update = {
                 s: {
                     a: 0.0
-                    for a in environment.p_S_prime_R_given_S_A
+                    for a in environment.p_S_prime_R_given_S_A[s]
                 }
                 for s in agent.pi
             }
