@@ -2,7 +2,10 @@ import importlib
 import inspect
 import os
 import pkgutil
-from typing import Set, Dict, List, Union
+import time
+from typing import Set, Dict, List, Union, Callable, Any
+
+import requests
 
 import rlai
 
@@ -10,7 +13,7 @@ import rlai
 def rl_text(
         chapter: Union[int, str],
         page: int
-):
+) -> Callable:
     """
     Decorator for RL text references.
 
@@ -20,7 +23,9 @@ def rl_text(
     :return: Decorator function.
     """
 
-    def inner(element):
+    def inner(
+            element: Any
+    ) -> Any:
 
         if os.getenv('ANNOTATIONS_ON', 'False') != 'True':
             return element
@@ -35,7 +40,7 @@ def rl_text(
 
 
 def summarize(
-        pkg,
+        pkg: Any,
         chapter_page_descriptions: Dict[int, Dict[int, List[str]]],
         paths_summarized: Set[str] = None,
 ):
@@ -72,11 +77,38 @@ def summarize(
                         if page not in chapter_page_descriptions[chapter]:
                             chapter_page_descriptions[chapter][page] = []
 
-                        line_no = inspect.findsource(attribute)[1] + 1
-                        src_path = f'{repo_src_url}{full_path.replace(".", "/").rsplit("/", maxsplit=1)[0]}.py#L{line_no}'
-                        chapter_page_descriptions[chapter][page].append(f'### [{full_path}]({src_path})\n```\n{attribute.__doc__.strip()}\n```\n')
+                        # inspect is unpredictable, in that it sometimes returns the correct line and sometimes returns
+                        # the line previous to the line with the attribute. check each.
+                        src_lines, line_no = inspect.findsource(attribute)
+                        if attribute.__name__ not in src_lines[line_no]:
+                            if attribute.__name__ in src_lines[line_no + 1]:
+                                line_no = line_no + 1
+                            else:
+                                raise ValueError('Attribute name not found on src lines.')
 
-                        paths_summarized.add(full_path)
+                        # inspect returns 0-based line number of decorator. move to next 1-based line for url.
+                        line_no += 1
+
+                        relative_url_filepath = full_path.replace(".", "/").rsplit("/", maxsplit=1)[0]
+                        if module.__file__.endswith('__init__.py'):
+                            relative_url_filepath = f'{relative_url_filepath}/__init__'
+
+                        # check url
+                        src_url = f'{repo_src_url}{relative_url_filepath}.py#L{line_no}'
+                        while True:
+                            response = requests.get(src_url)
+                            if response.status_code == 200:
+                                chapter_page_descriptions[chapter][page].append(f'### [{full_path}]({src_url})\n```\n{attribute.__doc__.strip()}\n```\n')
+                                paths_summarized.add(full_path)
+                                break
+                            elif response.status_code == 404:
+                                print(f'Invalid URL ({response.status_code}):  {src_url}')
+                                break
+                            elif response.status_code == 429:
+                                print(f'Rate limited ({response.status_code}):  Sleeping.')
+                                time.sleep(10)
+                            else:
+                                raise ValueError(f'Unexpected status code for URL {src_url}:  {response.status_code}')
 
         if module_is_pkg:
             summarize(module, chapter_page_descriptions, paths_summarized)
@@ -121,24 +153,24 @@ def main():
 
         meta_md.write(f'{readme_base}\n\n')
 
-        # write sorted string chapters
-        meta_md.write('# General Topics\n')
+        # write sorted-string chapters
+        meta_md.write('# Links to Code by Topic\n')
         for chapter in sorted(filter(lambda ch: isinstance(ch, str), chapter_page_descriptions)):
             ch_filename = f'ch_{chapter.replace(" ", "_")}.md'
             meta_md.write(f'### [{chapter}]({ch_filename})\n')
             with open(f'{docs_dir}{ch_filename}', 'w') as ch_md:
-                ch_md.write(f'# {chapter}\n')
+                ch_md.write(f'[Home](index.md) > {chapter}\n')
                 for page in sorted(chapter_page_descriptions[chapter]):
                     for description in sorted(chapter_page_descriptions[chapter][page]):
                         ch_md.write(description)
 
         # write sorted numeric chapters
-        meta_md.write('\n# Book Chapters\n')
+        meta_md.write('\n# Links to Code by Book Chapter\n')
         for chapter in sorted(filter(lambda ch: isinstance(ch, int), chapter_page_descriptions)):
             ch_filename = f'ch_{chapter}.md'
             meta_md.write(f'### [Chapter {chapter}:  {ch_num_name[chapter]}]({ch_filename})\n')
             with open(f'{docs_dir}{ch_filename}', 'w') as ch_md:
-                ch_md.write(f'# Chapter {chapter}:  {ch_num_name[chapter]}\n')
+                ch_md.write(f'[Home](index.md) > Chapter {chapter}:  {ch_num_name[chapter]}\n')
                 for page in sorted(chapter_page_descriptions[chapter]):
                     for description in sorted(chapter_page_descriptions[chapter][page]):
                         ch_md.write(description)
