@@ -166,8 +166,9 @@ class ContinuousActionPolicy(ParameterizedPolicy, ABC):
 
         state = dict(self.__dict__)
 
-        # the environment should not be pickled, as it won't be used when running or resuming the policy later, and it
-        # will contain references (e.g., to scatter plots) that will become invalid.
+        # certain environments cannot be pickled, and even if all could be pickled they wouldn't be used when running
+        # the policy later. this is because we always instantiate a new environment when starting the process. retaining
+        # the environment here would cause its references (e.g., to scatter plots) to be invalid.
         state['environment'] = None
 
         return state
@@ -219,7 +220,8 @@ class ContinuousActionNormalDistributionPolicy(ContinuousActionPolicy):
             self
     ):
         """
-        Commit updates that were previously appended with calls to `append_update`.
+        Commit updates that were previously appended with calls to `append_update`. Not intended to be called directly
+        by outside callers or inheritors.
         """
 
         # extract state-feature matrix
@@ -588,19 +590,20 @@ class ContinuousActionBetaDistributionPolicy(ContinuousActionPolicy):
 
         self.reset_action_scatter_plot_y_range()
 
-        if logging.getLogger().level <= logging.DEBUG:
+        if logging.getLogger().level <= logging.INFO:
             action_min_a = np.amin(self.action_theta_a, 1)
             action_max_a = np.amax(self.action_theta_a, 1)
             action_min_b = np.amin(self.action_theta_b, 1)
             action_max_b = np.amax(self.action_theta_b, 1)
+            logging.info(f'State dimensions:  {self.action_theta_a.shape[1]}')
             for i, (min_a, max_a, min_b, max_b) in enumerate(zip(action_min_a, action_max_a, action_min_b, action_max_b)):
-                logging.debug(f'Action {i} [min,max]:\n\ta:  [{min_a},{max_a}]\n\tb:  [{min_b},{max_b}]\n')
+                logging.info(f'Action {i} [min,max]:\n\ta:  [{min_a},{max_a}]\n\tb:  [{min_b},{max_b}]\n')
 
         # only output the hyperparameter table if we have a state-dimension name for each feature. some feature
         # extractors expand the feature space beyond the dimension names (e.g., via interaction terms), and we don't
         # have a good way to generate names for these extra features. such extractors also tend to generate large
         # feature spaces for which tabular output isn't easily readable.
-        if logging.getLogger().level <= logging.DEBUG and self.action_theta_a.shape[1] == len(self.environment.get_state_dimension_names()) + 1:
+        if logging.getLogger().level <= logging.INFO and self.action_theta_a.shape[1] == len(self.environment.get_state_dimension_names()) + 1:
             row_names = [
                 f'{action_name}_{p}'
                 for action_name in self.environment.get_action_dimension_names()
@@ -612,7 +615,7 @@ class ContinuousActionBetaDistributionPolicy(ContinuousActionPolicy):
                 for theta_a_row, theta_b_row in zip(self.action_theta_a, self.action_theta_b)
                 for row in [theta_a_row, theta_b_row]
             ], index=row_names, columns=col_names)
-            logging.debug(f'Per-action beta hyperparameters:\n{tabulate(theta_df, headers="keys", tablefmt="psql")}')
+            logging.info(f'Per-action beta hyperparameters:\n{tabulate(theta_df, headers="keys", tablefmt="psql")}')
 
     def reset_action_scatter_plot_y_range(
             self
@@ -772,13 +775,29 @@ class ContinuousActionBetaDistributionPolicy(ContinuousActionPolicy):
         action_a = np.exp(self.action_theta_a.dot(intercept_state_features))
         action_b = np.exp(self.action_theta_b.dot(intercept_state_features))
 
-        # sample each of the action dimensions and rescale
-        action_value = self.rescale(
-            np.array([
-                stats.beta.rvs(a=a, b=b, loc=0.0, scale=1.0, random_state=self.random_state)
-                for a, b in zip(action_a, action_b)
-            ])
-        )
+        try:
+
+            # sample each of the action dimensions and rescale
+            action_value = self.rescale(
+                np.array([
+                    stats.beta.rvs(a=a, b=b, loc=0.0, scale=1.0, random_state=self.random_state)
+                    for a, b in zip(action_a, action_b)
+                ])
+            )
+
+        # watch out for numerical issues (e.g., alpha step sizes that are too large and create issues for beta
+        # sampling). set a uniformly random action in such cases and report the error.
+        except ValueError as e:
+            if str(e) == 'Domain error in arguments.':
+                action_value = self.rescale(
+                    np.array([
+                        stats.beta.rvs(a=a, b=b, loc=0.0, scale=1.0, random_state=self.random_state)
+                        for a, b in zip(np.ones_like(action_a), np.ones_like(action_b))
+                    ])
+                )
+                logging.error(f'Caught {e} Setting action to {action_value}.')
+            else:
+                raise e
 
         action = ContinuousMultiDimensionalAction(
             value=action_value,
