@@ -498,7 +498,7 @@ class ApproximateStateActionValueEstimator(StateActionValueEstimator):
         self.experience_states: List[MdpState] = []
         self.experience_actions: List[Action] = []
         self.experience_values: List[float] = []
-        self.weights: np.ndarray = None
+        self.weights: Optional[np.ndarray] = None
         self.experience_pending: bool = False
 
     def __getitem__(
