@@ -1,3 +1,4 @@
+import logging
 import os
 import pickle
 import sys
@@ -9,6 +10,7 @@ from numpy.random import RandomState
 
 from rlai.gpi.utils import resume_from_checkpoint
 from rlai.meta import rl_text
+from rlai.policies.parameterized import ParameterizedPolicy
 from rlai.utils import (
     import_function,
     load_class,
@@ -49,76 +51,71 @@ def run(
     parser = get_argument_parser_for_run()
     parsed_args, unparsed_args = parse_arguments(parser, args)
 
+    if parsed_args.train_function is None:
+        raise ValueError('No training function specified. Cannot train.')
+
     if parsed_args.random_seed is None:
         warnings.warn('No random seed provided to the trainer. Results will not be replicable. Consider passing --random-seed argument.')
         random_state = RandomState()
     else:
         random_state = RandomState(parsed_args.random_seed)
 
+    # warn user, as training could take a long time and it'll be wasted effort if the agent is not saved.
+    if parsed_args.save_agent_path is None:
+        warnings.warn('No --save-agent-path has been specified, so no agent will be saved after training.')
+
+    # load training function and parse any arguments that it requires
+    train_function = import_function(parsed_args.train_function)
+    train_function_arg_parser = get_argument_parser_for_train_function(parsed_args.train_function)
+    parsed_train_function_args, unparsed_args = parse_arguments(train_function_arg_parser, unparsed_args)
+
     train_function_args = {
-        'thread_manager': thread_manager
+        'thread_manager': thread_manager,
+        **vars(parsed_train_function_args)
     }
 
+    # convert boolean strings to booleans
+    if train_function_args.get('update_upon_every_visit', None) is not None:
+        train_function_args['update_upon_every_visit'] = train_function_args['update_upon_every_visit'] == 'True'
+
+    if train_function_args.get('make_final_policy_greedy', None) is not None:
+        train_function_args['make_final_policy_greedy'] = train_function_args['make_final_policy_greedy'] == 'True'
+
+    if train_function_args.get('plot_state_value', None) is not None:
+        train_function_args['plot_state_value'] = train_function_args['plot_state_value'] == 'True'
+
     # load environment
-    if parsed_args.environment is not None:
-        environment_class = load_class(parsed_args.environment)
+    if train_function_args.get('environment', None) is not None:
+        environment_class = load_class(train_function_args['environment'])
         train_function_args['environment'], unparsed_args = environment_class.init_from_arguments(
             args=unparsed_args,
             random_state=random_state
         )
 
     # load planning environment
-    if parsed_args.planning_environment is not None:
-        planning_environment_class = load_class(parsed_args.planning_environment)
+    if train_function_args.get('planning_environment', None) is not None:
+        planning_environment_class = load_class(train_function_args['planning_environment'])
         train_function_args['planning_environment'], unparsed_args = planning_environment_class.init_from_arguments(
             args=unparsed_args,
             random_state=random_state
         )
-    else:
-        train_function_args['planning_environment'] = None
 
-    # get training function and parse its arguments
-    train_function = None
-    if parsed_args.train_function is not None:
-
-        train_function = import_function(parsed_args.train_function)
-        train_function_arg_parser = get_argument_parser_for_train_function(parsed_args.train_function)
-        parsed_train_function_args, unparsed_args = parse_arguments(train_function_arg_parser, unparsed_args)
-
-        # convert parsed boolean strings to bools
-        if hasattr(parsed_train_function_args, 'update_upon_every_visit') and parsed_train_function_args.update_upon_every_visit is not None:
-            parsed_train_function_args.update_upon_every_visit = parsed_train_function_args.update_upon_every_visit == 'True'
-
-        if hasattr(parsed_train_function_args, 'make_final_policy_greedy') and parsed_train_function_args.make_final_policy_greedy is not None:
-            parsed_train_function_args.make_final_policy_greedy = parsed_train_function_args.make_final_policy_greedy == 'True'
-
-        train_function_args.update(vars(parsed_train_function_args))
-
-        # initialize state-action value estimator if one is given
-        if hasattr(parsed_train_function_args, 'q_S_A') and parsed_train_function_args.q_S_A is not None:
-            estimator_class = load_class(parsed_train_function_args.q_S_A)
-            train_function_args['q_S_A'], unparsed_args = estimator_class.init_from_arguments(
-                unparsed_args,
-                random_state=random_state,
-                environment=train_function_args['environment']
-            )
-
-    agent = None
-
-    if parsed_args.agent is not None:
-
-        agent_class = load_class(parsed_args.agent)
+    # load agent
+    if train_function_args.get('agent', None) is not None:
+        agent_class = load_class(train_function_args['agent'])
         agents, unparsed_args = agent_class.init_from_arguments(
             args=unparsed_args,
             random_state=random_state,
-            pi=None if train_function_args.get('q_S_A') is None else train_function_args['q_S_A'].get_initial_policy()
+            environment=train_function_args['environment']
         )
 
-        if len(agents) != 1:  # pragma no cover
-            raise ValueError('Training is only supported for single agents. Please specify one agent.')
+        if len(agents) != 1:
+            raise Exception(f'Expected exactly 1 agent, but got {len(agents)}.')
 
         agent = agents[0]
         train_function_args['agent'] = agent
+    else:
+        agent = None
 
     if '--help' in unparsed_args:
         unparsed_args.remove('--help')
@@ -126,52 +123,49 @@ def run(
     if len(unparsed_args) > 0:
         raise ValueError(f'Unparsed arguments remain:  {unparsed_args}')
 
-    if train_function is None:
-        warnings.warn('No training function specified. Cannot train.')
+    new_checkpoint_path = None
+
+    # resumption will return a trained version of the agent contained in the checkpoint file
+    if parsed_args.resume:
+        agent = resume_from_checkpoint(
+            resume_function=train_function,
+            **train_function_args
+        )
+
+    # fresh training will train the agent that was initialized above and passed in
     else:
 
-        # warn user now, as training could take a long time and it'll be wasted effort if the agent is not saved.
-        if parsed_args.save_agent_path is None:
-            warnings.warn('No --save-agent-path has been specified, so no agent will be saved after training.')
+        if train_function_args_callback is not None:
+            train_function_args_callback(train_function_args)
 
-        # resumption will return a trained version of the agent contained in the checkpoint file
-        if parsed_args.resume:
-            agent = resume_from_checkpoint(
-                resume_function=train_function,
-                **train_function_args
-            )
+        new_checkpoint_path = train_function(
+            **train_function_args
+        )
 
-        # fresh training will train the agent that was initialized above and passed in
-        else:
+        train_function_args['environment'].close()
 
-            if train_function_args_callback is not None:
-                train_function_args_callback(train_function_args)
+        if isinstance(agent.pi, ParameterizedPolicy):
+            agent.pi.close()
 
-            train_function(
-                **train_function_args
-            )
+    logging.info('Training complete.')
 
-            train_function_args['environment'].close()
+    # try to save agent
+    if agent is None:  # pragma no cover
+        warnings.warn('No agent resulting at end of training. Nothing to save.')
+    elif parsed_args.save_agent_path is None:
+        warnings.warn('No --save-agent-path specified. Not saving agent.')
+    else:
+        with open(os.path.expanduser(parsed_args.save_agent_path), 'wb') as f:
+            pickle.dump(agent, f)
 
-        print('Training complete.')
+        logging.info(f'Saved agent to {parsed_args.save_agent_path}')
 
-        # try to save agent
-        if agent is None:  # pragma no cover
-            warnings.warn('No agent resulting at end of training. Nothing to save.')
-        elif parsed_args.save_agent_path is None:
-            warnings.warn('No --save-agent-path specified. Not saving agent.')
-        else:
-            with open(os.path.expanduser(parsed_args.save_agent_path), 'wb') as f:
-                pickle.dump(agent, f)
-
-            print(f'Saved agent to {parsed_args.save_agent_path}')
-
-    return train_function_args.get('checkpoint_path'), parsed_args.save_agent_path
+    return new_checkpoint_path, parsed_args.save_agent_path
 
 
 def get_argument_parser_for_run() -> ArgumentParser:
     """
-    Get argument parser for the run function.
+    Get argument parser for values used in the run function.
 
     :return: Argument parser.
     """
@@ -181,24 +175,6 @@ def get_argument_parser_for_run() -> ArgumentParser:
         description='Train an agent in an environment.'
     )
 
-    parser.add_argument(
-        '--agent',
-        type=str,
-        help='Fully-qualified type name of agent to train.'
-    )
-
-    parser.add_argument(
-        '--environment',
-        type=str,
-        help='Fully-qualified type name of environment to train agent in.'
-    )
-
-    parser.add_argument(
-        '--planning-environment',
-        type=str,
-        help='Fully-qualified type name of planning environment to train agent in.'
-    )
-
     parser.add_argument(
         '--train-function',
         type=str,
@@ -240,11 +216,11 @@ def get_argument_parser_for_train_function(
 
     function = import_function(function_name)
 
-    # get argument names expected by training function
+    # get argument names defined by the specified training function
     # noinspection PyUnresolvedReferences
-    arg_names = function.__code__.co_varnames[:function.__code__.co_argcount]
+    function_arg_names = function.__code__.co_varnames[:function.__code__.co_argcount]
 
-    def filter_add_argument(
+    def add_argument(
             name: str,
             **kwargs
     ):
@@ -256,92 +232,149 @@ def get_argument_parser_for_train_function(
         """
 
         var_name = name.lstrip('-').replace('-', '_')
-        if var_name in arg_names:
+        if var_name in function_arg_names:
             argument_parser.add_argument(
                 name,
                 **kwargs
             )
 
-    filter_add_argument(
+    # add the superset of all arguments used across all training function. the filter will only retain those allowed.
+
+    add_argument(
+        '--agent',
+        type=str,
+        help='Fully-qualified type name of agent to train.'
+    )
+
+    add_argument(
+        '--environment',
+        type=str,
+        help='Fully-qualified type name of environment to train agent in.'
+    )
+
+    add_argument(
+        '--planning-environment',
+        type=str,
+        help='Fully-qualified type name of planning environment to train agent in.'
+    )
+
+    add_argument(
         '--num-improvements',
         type=int,
         help='Number of improvements.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--num-episodes-per-improvement',
         type=int,
         help='Number of episodes per improvement.'
     )
 
-    filter_add_argument(
+    add_argument(
+        '--num-episodes',
+        type=int,
+        help='Number of episodes.'
+    )
+
+    add_argument(
         '--num-updates-per-improvement',
         type=int,
         help='Number of state-action value updates per policy improvement.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--update-upon-every-visit',
         type=str,
         choices=['True', 'False'],
         help='Whether or not to update values upon each visit to a state or state-action pair.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--alpha',
         type=float,
         help='Step size.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--make-final-policy-greedy',
         type=str,
         choices=['True', 'False'],
         help='Whether or not to make the final policy greedy after training is complete.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--num-improvements-per-plot',
         type=int,
         help='Number of improvements per plot.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--num-improvements-per-checkpoint',
         type=int,
         help='Number of improvements per checkpoint.'
     )
 
-    filter_add_argument(
+    add_argument(
+        '--num-episodes-per-checkpoint',
+        type=int,
+        help='Number of episodes per checkpoint.'
+    )
+
+    add_argument(
         '--checkpoint-path',
         type=str,
         help='Path to checkpoint file.'
     )
 
-    filter_add_argument(
+    add_argument(
         '--mode',
         type=str,
         help='Temporal difference evaluation mode (SARSA, Q_LEARNING, EXPECTED_SARSA).'
     )
 
-    filter_add_argument(
+    add_argument(
         '--n-steps',
         type=int,
         help='N-step update value.'
     )
 
-    filter_add_argument(
-        '--q-S-A',
-        type=str,
-        help='Fully-qualified type name of state-action value estimator to use.'
-    )
-
-    filter_add_argument(
+    add_argument(
         '--pdf-save-path',
         type=str,
         help='Path where a PDF of all plots is to be saved.'
     )
 
+    add_argument(
+        '--plot-state-value',
+        type=str,
+        choices=['True', 'False'],
+        help='Whether or not to plot the state value.'
+    )
+
+    add_argument(
+        '--training-pool-directory',
+        type=str,
+        help='Path to directory in which to store pooled training runs.'
+    )
+
+    add_argument(
+        '--training-pool-count',
+        type=int,
+        help='Number of runners in the training pool.'
+    )
+
+    add_argument(
+        '--training-pool-iterate-episodes',
+        type=int,
+        help='Number of episodes per training pool iteration.'
+    )
+
+    add_argument(
+        '--training-pool-evaluate-episodes',
+        type=int,
+        help='Number of episodes to evaluate the agent when iterating the training pool.'
+    )
+
     return argument_parser
 
 
