@@ -47,7 +47,7 @@ class ApproximateValueEstimator(ValueEstimator):
 
     def __init__(
             self,
-            estimator,
+            estimator: 'ApproximateStateValueEstimator',
             state: MdpState
     ):
         """
@@ -57,14 +57,14 @@ class ApproximateValueEstimator(ValueEstimator):
         :param state: State.
         """
 
-        self.estimator: ApproximateStateValueEstimator = estimator
+        self.estimator = estimator
         self.state = state
 
 
 @rl_text(chapter='Value Estimation', page=195)
 class ApproximateStateValueEstimator(StateValueEstimator):
     """
-    Approximate state value estimator.
+    Approximate state-value estimator.
     """
 
     @classmethod
@@ -106,12 +106,12 @@ class ApproximateStateValueEstimator(StateValueEstimator):
             environment: MdpEnvironment
     ) -> Tuple[StateValueEstimator, List[str]]:
         """
-        Initialize a state-action value estimator from arguments.
+        Initialize a state-value estimator from arguments.
 
         :param args: Arguments.
         :param random_state: Random state.
         :param environment: Environment.
-        :return: 2-tuple of a state-action value estimator and a list of unparsed arguments.
+        :return: 2-tuple of a state-value estimator and a list of unparsed arguments.
         """
 
         parsed_args, unparsed_args = parse_arguments(cls, args)
@@ -195,6 +195,15 @@ class ApproximateStateValueEstimator(StateValueEstimator):
             self.weights = None
             self.experience_pending = False
 
+    def reset(
+            self
+    ):
+        """
+        Reset the estimator.
+        """
+
+        self.model.reset()
+
     def evaluate(
             self,
             state: MdpState
@@ -252,7 +261,7 @@ class ApproximateStateValueEstimator(StateValueEstimator):
 
         self.experience_states: List[MdpState] = []
         self.experience_values: List[float] = []
-        self.weights: np.ndarray = None
+        self.weights: Optional[np.ndarray] = None
         self.experience_pending: bool = False
 
     def __getitem__(
@@ -288,14 +297,14 @@ class ApproximateStateValueEstimator(StateValueEstimator):
         Check whether a state is defined by the estimator.
 
         :param state: State.
-        :return: True if defined and False otherise.
+        :return: True if defined and False otherwise.
         """
 
         return True
 
     def __eq__(
             self,
-            other
+            other: object
     ) -> bool:
         """
         Check whether the estimator equals another.
@@ -304,13 +313,14 @@ class ApproximateStateValueEstimator(StateValueEstimator):
         :return: True if equal and False otherwise.
         """
 
-        other: ApproximateStateValueEstimator
+        if not isinstance(other, ApproximateStateValueEstimator):
+            raise ValueError(f'Expected {ApproximateStateValueEstimator}')
 
         return self.model == other.model
 
     def __ne__(
             self,
-            other
+            other: object
     ) -> bool:
         """
         Check whether the estimator does not equal another.
